#if defined(__dsPIC33F__)
#include "p33Fxxxx.h"
#elif defined(__PIC24H__)
#include "p24Hxxxx.h"
#endif

#include "i2sDrv.h"
//#include "tglPin.h"

unsigned int dciRxBuffA;
unsigned int dciTxBuffA;
/*=============================================================================
i2sInit(): Initialise DCI peripheral for I2S Interface
=============================================================================*/

void i2sInit (void)
{

/*
In this section we will set up the DCI module for I2S operation to interface 
with a DIR9001 SPDIF decoder receiving data at 44.1kHz STEREO The timing diagram is 
provided in Fig 1 below:
                                  FIGURE 1
      _________________________________

      |		 	    	              |	                              |
COFS: |	  	     	                  |_______________________________|
       _   _   _   _   _   _   _   _   _   _   _   _   _   _   _   _   _    
CSCK:_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|
          |<--------Right Channel Data--->|<----Left Channel Data-------->|
          |<---16bits---->|               |<---16bits---->|               |
          |<---TXBUF0---->|               |<---TXBUF0---->|               |
          |<---RXBUF0---->|               |<---RXBUF0---->|               |
          |<--TimeSlot0-->|               |<--TimeSlot0-->|               |
          |<-----------1 Frame----------->|<-----------1 Frame----------->|
          |<--------------------1/Fs = 22.67 microseconds---------------->|
*/

//DCI Control Register DCICON1 Initialization
	DCICON1bits.CSCKD 	= 1;	// Serial Bit Clock (CSCK pin) is input	DIR9001 controls data transfer
	DCICON1bits.CSCKE 	= 1;	// Data changes on falling edge sampled on rising edge of CSCK, from DIR9001 data sheet
	DCICON1bits.COFSD	= 1;	// Frame Sync Signal is input DIR9001 controls data transfer
	DCICON1bits.UNFM 	= 0;	// Transmit 0s on a transmit underflow
	DCICON1bits.CSDOM 	= 1;	// CSDO pin tristated during disabled transmit time slots, CSDO needs an EXTERNAL pull-down resistor
	DCICON1bits.DJST 	= 0;	// Data TX/RX is begun one serial clock cycle after frame sync pulse
	DCICON1bits.COFSM 	= 1;	// Frame Sync Signal set up for I2S mode 
	DCICON1bits.DLOOP 	= 0;	// No Loopback

// DCI Control Register DCICON2 Initialization
	DCICON2bits.BLEN	= 0;	// One data word will be buffered between interrupts	
	DCICON2bits.COFSG 	= 0;	// Data frame has 1 word
	DCICON2bits.WS 		= 15;	// Data word size is 16 bits


// DCI Control Register DCICON3 Initialization
	DCICON3 = 0;	    // Set up CSCK Bit Clock Frequency (disabled)

// Transmit Slot Control Register Initialization
	TSCONbits.TSE0 = 1;	    // Transmit on Time Slot 0	
	
//Receiver Slot Control Register Initialization
	RSCONbits.RSE0 = 1;	    // Receive on Time Slot 0	

// Disable DCI Interrupt and Enable DCI module
	IFS3bits.DCIIF=0;  //DCIIF: DCI Event Interrupt Flag Status bit
    IEC3bits.DCIIE=1;  //DCIIE: DCI Event Interrupt Enable bit

	DCICON1bits.DCIEN = 1; 

}

/*=============================================================================
DCIInterrupt(): DCI Interrupt handler
=============================================================================*/
void __attribute__((__interrupt__)) _DCIInterrupt(void)
{
	_RA2 = 1;

	TXBUF0=RXBUF0;
	//TXBUF0=0b1101011100000011; //useful for testing purposes
	IFS3bits.DCIIF=0;  //DCIIF: DCI Event Interrupt Flag Status bit, clear the Interrupt flag
	_RA2 = 0;
}

/*=============================================================================
main(): main function
=============================================================================*/
main ()
{	
// Configure Oscillator to operate the device at 60Mhz
// Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
// Fosc= 7.37*33/(2*2)=60Mhz for 7.37 input clock
	PLLFBD=31;					// M=33
	CLKDIVbits.PLLPOST=0;		// N1=2
	CLKDIVbits.PLLPRE=0;		// N2=2
	OSCTUN=0;					// Tune FRC oscillator, if FRC is used

// Disable Watch Dog Timer
    RCONbits.SWDTEN=0;

// Clock switch to incorporate PLL
	__builtin_write_OSCCONH(0x01);		// Initiate Clock Switch to
										// FRC with PLL (NOSC=0b001)
	__builtin_write_OSCCONL(0x01);		// Start clock switching
	while (OSCCONbits.COSC != 0b001);	// Wait for Clock switch to occur	

// Wait for PLL to lock
	while(OSCCONbits.LOCK!=1) {};

// Init DCI for I2S interface
   
	i2sInit();
	TRISA = 0xff80;	//set for debugging purposes


	// main loop    
	while( 1)
	{   
   	//_RA2 = 1;

	//_RA2 = 0;
	} // main loop
}

